#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>

#include "uartdrv.h"
#include "debug.h"


volatile uartdrvCmdBuf_t uartdrvCmdBuf;




void uartdrvInit() {

  // UART
  //      TXD
  DDRD |= (1 << PD1);

  //        transmitter enable
  UCSRB = (1 << TXEN) | (1 << RXEN) | (1 << RXCIE);
  //        8 data bit, no parity, 1 stop bit
  UCSRC = (1 << UCSZ1) | (1 << UCSZ0);
  //        19200 Baud @ 16MHz
  UBRRL = 51;
  UBRRH = 0;
  FILE *mystdout = fdevopen(uartdrvPutchar, NULL);
  stdout = mystdout;

  uartdrvCmdBuf.idx = 0;
  uartdrvCmdBuf.cmd = 0;
  uartdrvCmdBuf.cmdStart = 0;
  uartdrvCmdBuf.cmdDelay = 0;
  uartdrvCmdBuf.cmdReady = 0;
}

void uartdrvWrite(uint8_t o) {
  loop_until_bit_is_clear(PIND, PD2);
  loop_until_bit_is_set(UCSRA, UDRE);
  UDR = o;
}


int uartdrvPutchar(char c, FILE *stream) {
  if (c == '\n')
    uartdrvPutchar('\r', stream);
  uartdrvWrite((uint8_t) c);
  return 0;
}

void uartdrvCmdCycle() {
  if (uartdrvCmdBuf.cmdStart == 1) {
    uartdrvCmdBuf.cmdDelay++;
    if (uartdrvCmdBuf.cmdDelay > UART_CMD_CYCLES) {
      uartdrvCmdBuf.idx = 0;
      uartdrvCmdBuf.cmdStart = 0;
      uartdrvCmdBuf.cmdDelay= 0;
      // myPrintf("clear invalid cmd\n");
    }
  }
}

void uartdrvCmdReceived() {
  uartdrvWrite(0x7f);
  uartdrvWrite(uartdrvCmdBuf.cmd);
  uartdrvWrite(0);
  uartdrvWrite(0);
  uartdrvWrite(0);
}

ISR(USART_RXC_vect) {
  uint8_t c = UDR;
  if (uartdrvCmdBuf.cmdReady != 1) {
    if (uartdrvCmdBuf.idx == 0) {
      uartdrvCmdBuf.cmdStart = 1;
      uartdrvCmdBuf.cmd = c;
      uartdrvCmdBuf.idx++;
    } else if (uartdrvCmdBuf.idx >= 1 && uartdrvCmdBuf.idx <= UART_CMD_BUF_SIZE) {
      uartdrvCmdBuf.buf[uartdrvCmdBuf.idx - 1] = c;
      uartdrvCmdBuf.idx++;
    }

    if (uartdrvCmdBuf.idx == (UART_CMD_BUF_SIZE + 1)) {

      uartdrvCmdBuf.idx = 0;
      uartdrvCmdBuf.cmdStart = 0;
      uartdrvCmdBuf.cmdDelay = 0;
      uartdrvCmdBuf.cmdReady = 1;
    }
  }
}


